@@ -126,15 +126,12 @@ class BaseTestCase(base.TestCase):
         self.cloud_config = self.config.get_one_cloud(
             cloud=test_cloud, validate=False)
         self.cloud = shade.OpenStackCloud(
-            cloud_config=self.cloud_config,
-            log_inner_exceptions=True)
+            cloud_config=self.cloud_config)
         self.strict_cloud = shade.OpenStackCloud(
             cloud_config=self.cloud_config,
-            log_inner_exceptions=True,
             strict=True)
         self.op_cloud = shade.OperatorCloud(
-            cloud_config=self.cloud_config,
-            log_inner_exceptions=True)
+            cloud_config=self.cloud_config)
 
 
 class TestCase(BaseTestCase):
@@ -259,9 +256,11 @@ class RequestsMockTestCase(BaseTestCase):
             response['enabled'] = enabled
             request['enabled'] = enabled
         response.setdefault('enabled', True)
+        request.setdefault('enabled', True)
         if description:
             response['description'] = description
             request['description'] = description
+        request.setdefault('description', None)
         if v3:
             project_key = 'project'
         else:
@@ -275,7 +274,7 @@ class RequestsMockTestCase(BaseTestCase):
         name = name or self.getUniqueString('groupname')
         domain_id = uuid.UUID(domain_id or uuid.uuid4().hex).hex
         response = {'id': group_id, 'name': name, 'domain_id': domain_id}
-        request = {'name': name}
+        request = {'name': name, 'domain_id': domain_id}
         if description is not None:
             response['description'] = description
             request['description'] = description
@@ -290,11 +289,12 @@ class RequestsMockTestCase(BaseTestCase):
         user_id = uuid.uuid4().hex
 
         response = {'name': name, 'id': user_id}
-        request = {'name': name, 'password': password, 'tenantId': None}
+        request = {'name': name, 'password': password}
 
         if kwargs.get('domain_id'):
             kwargs['domain_id'] = uuid.UUID(kwargs['domain_id']).hex
             response['domain_id'] = kwargs.pop('domain_id')
+            request['domain_id'] = response['domain_id']
 
         response['email'] = kwargs.pop('email', None)
         request['email'] = response['email']
@@ -373,6 +373,8 @@ class RequestsMockTestCase(BaseTestCase):
         response = {'id': endpoint_id, 'service_id': service_id,
                     'region': region}
         v3_endpoints = {}
+        request = response.copy()
+        request.pop('id')
         if admin_url:
             response['adminURL'] = admin_url
             v3_endpoints['admin'] = self._get_endpoint_v3_data(
@@ -387,6 +389,9 @@ class RequestsMockTestCase(BaseTestCase):
                 service_id, region, public_url, interface='public')
         request = response.copy()
         request.pop('id')
+        for u in ('publicURL', 'internalURL', 'adminURL'):
+            if request.get(u):
+                request[u.lower()] = request.pop(u)
         return _EndpointDataV2(endpoint_id, service_id, region, public_url,
                                internal_url, admin_url, v3_endpoints,
                                {'endpoint': response}, {'endpoint': request})
@@ -400,6 +405,10 @@ class RequestsMockTestCase(BaseTestCase):
         return _RoleData(role_id, role_name, {'role': response},
                          {'role': request})
 
+    def use_nothing(self):
+        self.calls = []
+        self._uri_registry.clear()
+
     def use_keystone_v3(self, catalog='catalog-v3.json'):
         self.adapter = self.useFixture(rm_fixture.Fixture())
         self.calls = []
@@ -439,17 +448,17 @@ class RequestsMockTestCase(BaseTestCase):
         self.cloud_config = self.config.get_one_cloud(
             cloud=test_cloud, validate=True, **kwargs)
         self.cloud = shade.OpenStackCloud(
-            cloud_config=self.cloud_config,
-            log_inner_exceptions=True)
+            cloud_config=self.cloud_config)
         self.op_cloud = shade.OperatorCloud(
-            cloud_config=self.cloud_config,
-            log_inner_exceptions=True)
+            cloud_config=self.cloud_config)
 
     def get_glance_discovery_mock_dict(
-            self, image_version_json='image-version.json'):
+            self,
+            image_version_json='image-version.json',
+            image_discovery_url='https://image.example.com/'):
         discovery_fixture = os.path.join(
             self.fixtures_directory, image_version_json)
-        return dict(method='GET', uri='https://image.example.com/',
+        return dict(method='GET', uri=image_discovery_url,
                     status_code=300,
                     text=open(discovery_fixture, 'r').read())
 
@@ -459,14 +468,23 @@ class RequestsMockTestCase(BaseTestCase):
         return dict(method='GET', uri="https://dns.example.com/",
                     text=open(discovery_fixture, 'r').read())
 
-    def use_glance(self, image_version_json='image-version.json'):
+    def get_ironic_discovery_mock_dict(self):
+        discovery_fixture = os.path.join(
+            self.fixtures_directory, "baremetal.json")
+        return dict(method='GET', uri="https://bare-metal.example.com/",
+                    text=open(discovery_fixture, 'r').read())
+
+    def use_glance(
+            self, image_version_json='image-version.json',
+            image_discovery_url='https://image.example.com/'):
         # NOTE(notmorgan): This method is only meant to be used in "setUp"
         # where the ordering of the url being registered is tightly controlled
         # if the functionality of .use_glance is meant to be used during an
         # actual test case, use .get_glance_discovery_mock and apply to the
         # right location in the mock_uris when calling .register_uris
         self.__do_register_uris([
-            self.get_glance_discovery_mock_dict(image_version_json)])
+            self.get_glance_discovery_mock_dict(
+                image_version_json, image_discovery_url)])
 
     def use_designate(self):
         # NOTE(slaweq): This method is only meant to be used in "setUp"
@@ -477,6 +495,15 @@ class RequestsMockTestCase(BaseTestCase):
         self.__do_register_uris([
             self.get_designate_discovery_mock_dict()])
 
+    def use_ironic(self):
+        # NOTE(TheJulia): This method is only meant to be used in "setUp"
+        # where the ordering of the url being registered is tightly controlled
+        # if the functionality of .use_ironic is meant to be used during an
+        # actual test case, use .get_ironic_discovery_mock and apply to the
+        # right location in the mock_uris when calling .register_uris
+        self.__do_register_uris([
+            self.get_ironic_discovery_mock_dict()])
+
     def register_uris(self, uri_mock_list=None):
         """Mock a list of URIs and responses via requests mock.
 
@@ -528,6 +555,12 @@ class RequestsMockTestCase(BaseTestCase):
             key = '{method}|{uri}|{params}'.format(
                 method=method, uri=uri, params=kw_params)
             validate = to_mock.pop('validate', {})
+            valid_keys = set(['json', 'headers', 'params'])
+            invalid_keys = set(validate.keys()) - valid_keys
+            if invalid_keys:
+                raise TypeError(
+                    "Invalid values passed to validate: {keys}".format(
+                        keys=invalid_keys))
             headers = structures.CaseInsensitiveDict(to_mock.pop('headers',
                                                                  {}))
             if 'content-type' not in headers:
@@ -600,3 +633,17 @@ class RequestsMockTestCase(BaseTestCase):
         if do_count:
             self.assertEqual(
                 len(self.calls), len(self.adapter.request_history))
+
+
+class IronicTestCase(RequestsMockTestCase):
+
+    def setUp(self):
+        super(IronicTestCase, self).setUp()
+        self.use_ironic()
+        self.uuid = str(uuid.uuid4())
+        self.name = self.getUniqueString('name')
+
+    def get_mock_url(self, resource=None, append=None, qs_elements=None):
+        return super(IronicTestCase, self).get_mock_url(
+            service_type='baremetal', interface='public', resource=resource,
+            append=append, base_url_append='v1', qs_elements=qs_elements)
