@@ -12,16 +12,19 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+# TODO(mordred) There are mocks of the image_client in here that are not
+#               using requests_mock. Erradicate them.
+
 import operator
 import tempfile
 import uuid
 
 import mock
 import munch
-import os_client_config as occ
 import six
 
 import shade
+import shade.openstackcloud
 from shade import exc
 from shade import meta
 from shade.tests import fakes
@@ -42,6 +45,8 @@ class BaseTestImage(base.RequestsMockTestCase):
         self.fake_image_dict = fakes.make_fake_image(image_id=self.image_id)
         self.fake_search_return = {'images': [self.fake_image_dict]}
         self.output = uuid.uuid4().bytes
+        self.image_name = self.getUniqueString('image')
+        self.container_name = self.getUniqueString('container')
 
 
 class TestImage(BaseTestImage):
@@ -136,6 +141,58 @@ class TestImage(BaseTestImage):
             self.cloud.list_images())
         self.assert_calls()
 
+    def test_list_images_show_all(self):
+        self.register_uris([
+            dict(method='GET',
+                 uri='https://image.example.com/v2/images?member_status=all',
+                 json=self.fake_search_return)
+        ])
+        self.assertEqual(
+            self.cloud._normalize_images([self.fake_image_dict]),
+            self.cloud.list_images(show_all=True))
+        self.assert_calls()
+
+    def test_list_images_show_all_deleted(self):
+        deleted_image = self.fake_image_dict.copy()
+        deleted_image['status'] = 'deleted'
+        self.register_uris([
+            dict(method='GET',
+                 uri='https://image.example.com/v2/images?member_status=all',
+                 json={'images': [self.fake_image_dict, deleted_image]})
+        ])
+        self.assertEqual(
+            self.cloud._normalize_images([
+                self.fake_image_dict, deleted_image]),
+            self.cloud.list_images(show_all=True))
+        self.assert_calls()
+
+    def test_list_images_no_filter_deleted(self):
+        deleted_image = self.fake_image_dict.copy()
+        deleted_image['status'] = 'deleted'
+        self.register_uris([
+            dict(method='GET',
+                 uri='https://image.example.com/v2/images',
+                 json={'images': [self.fake_image_dict, deleted_image]})
+        ])
+        self.assertEqual(
+            self.cloud._normalize_images([
+                self.fake_image_dict, deleted_image]),
+            self.cloud.list_images(filter_deleted=False))
+        self.assert_calls()
+
+    def test_list_images_filter_deleted(self):
+        deleted_image = self.fake_image_dict.copy()
+        deleted_image['status'] = 'deleted'
+        self.register_uris([
+            dict(method='GET',
+                 uri='https://image.example.com/v2/images',
+                 json={'images': [self.fake_image_dict, deleted_image]})
+        ])
+        self.assertEqual(
+            self.cloud._normalize_images([self.fake_image_dict]),
+            self.cloud.list_images())
+        self.assert_calls()
+
     def test_list_images_string_properties(self):
         image_dict = self.fake_image_dict.copy()
         image_dict['properties'] = 'list,of,properties'
@@ -204,8 +261,6 @@ class TestImage(BaseTestImage):
 
     def test_create_image_task(self):
         self.cloud.image_api_use_tasks = True
-        image_name = 'name-99'
-        container_name = 'image_upload_v2_test_container'
         endpoint = self.cloud._object_store_client.get_endpoint()
 
         task_id = str(uuid.uuid4())
@@ -232,18 +287,18 @@ class TestImage(BaseTestImage):
                      slo={'min_segment_size': 500})),
             dict(method='HEAD',
                  uri='{endpoint}/{container}'.format(
-                     endpoint=endpoint, container=container_name),
+                     endpoint=endpoint, container=self.container_name),
                  status_code=404),
             dict(method='PUT',
                  uri='{endpoint}/{container}'.format(
-                     endpoint=endpoint, container=container_name),
+                     endpoint=endpoint, container=self.container_name),
                  status_code=201,
                  headers={'Date': 'Fri, 16 Dec 2016 18:21:20 GMT',
                           'Content-Length': '0',
                           'Content-Type': 'text/html; charset=UTF-8'}),
             dict(method='HEAD',
                  uri='{endpoint}/{container}'.format(
-                     endpoint=endpoint, container=container_name),
+                     endpoint=endpoint, container=self.container_name),
                  headers={'Content-Length': '0',
                           'X-Container-Object-Count': '0',
                           'Accept-Ranges': 'bytes',
@@ -255,13 +310,13 @@ class TestImage(BaseTestImage):
                          'Content-Type': 'text/plain; charset=utf-8'}),
             dict(method='HEAD',
                  uri='{endpoint}/{container}/{object}'.format(
-                     endpoint=endpoint, container=container_name,
-                     object=image_name),
+                     endpoint=endpoint, container=self.container_name,
+                     object=self.image_name),
                  status_code=404),
             dict(method='PUT',
                  uri='{endpoint}/{container}/{object}'.format(
-                     endpoint=endpoint, container=container_name,
-                     object=image_name),
+                     endpoint=endpoint, container=self.container_name,
+                     object=self.image_name),
                  status_code=201,
                  validate=dict(
                      headers={'x-object-meta-x-shade-md5': fakes.NO_MD5,
@@ -275,8 +330,9 @@ class TestImage(BaseTestImage):
                      json=dict(
                          type='import', input={
                              'import_from': '{container}/{object}'.format(
-                                 container=container_name, object=image_name),
-                             'image_properties': {'name': image_name}}))
+                                 container=self.container_name,
+                                 object=self.image_name),
+                             'image_properties': {'name': self.image_name}}))
                  ),
             dict(method='GET',
                  uri='https://image.example.com/v2/tasks/{id}'.format(
@@ -285,7 +341,7 @@ class TestImage(BaseTestImage):
             dict(method='GET',
                  uri='https://image.example.com/v2/tasks/{id}'.format(
                      id=task_id),
-                 json={'images': args}),
+                 json=args),
             dict(method='GET', uri='https://image.example.com/v2/images',
                  json={'images': [image_no_checksums]}),
             dict(method='PATCH',
@@ -294,8 +350,8 @@ class TestImage(BaseTestImage):
                  validate=dict(
                      json=sorted([{u'op': u'add',
                                    u'value': '{container}/{object}'.format(
-                                       container=container_name,
-                                       object=image_name),
+                                       container=self.container_name,
+                                       object=self.image_name),
                                    u'path': u'/owner_specified.shade.object'},
                                   {u'op': u'add', u'value': fakes.NO_MD5,
                                    u'path': u'/owner_specified.shade.md5'},
@@ -306,13 +362,111 @@ class TestImage(BaseTestImage):
                          'Content-Type':
                              'application/openstack-images-v2.1-json-patch'})
                  ),
+            dict(method='HEAD',
+                 uri='{endpoint}/{container}/{object}'.format(
+                     endpoint=endpoint, container=self.container_name,
+                     object=self.image_name),
+                 headers={
+                     'X-Timestamp': '1429036140.50253',
+                     'X-Trans-Id': 'txbbb825960a3243b49a36f-005a0dadaedfw1',
+                     'Content-Length': '1290170880',
+                     'Last-Modified': 'Tue, 14 Apr 2015 18:29:01 GMT',
+                     'X-Object-Meta-X-Shade-Sha256': fakes.NO_SHA256,
+                     'X-Object-Meta-X-Shade-Md5': fakes.NO_MD5,
+                     'Date': 'Thu, 16 Nov 2017 15:24:30 GMT',
+                     'Accept-Ranges': 'bytes',
+                     'Content-Type': 'application/octet-stream',
+                     'Etag': fakes.NO_MD5}),
+            dict(method='DELETE',
+                 uri='{endpoint}/{container}/{object}'.format(
+                     endpoint=endpoint, container=self.container_name,
+                     object=self.image_name)),
             dict(method='GET', uri='https://image.example.com/v2/images',
                  json=self.fake_search_return)
         ])
 
         self.cloud.create_image(
-            image_name, self.imagefile.name, wait=True, timeout=1,
-            is_public=False, container=container_name)
+            self.image_name, self.imagefile.name, wait=True, timeout=1,
+            is_public=False, container=self.container_name)
+
+        self.assert_calls()
+
+    def test_delete_autocreated_no_tasks(self):
+        self.use_nothing()
+        self.cloud.image_api_use_tasks = False
+        deleted = self.cloud.delete_autocreated_image_objects(
+            container=self.container_name)
+        self.assertFalse(deleted)
+        self.assert_calls()
+
+    def test_delete_autocreated_image_objects(self):
+        self.use_keystone_v3()
+        self.cloud.image_api_use_tasks = True
+        endpoint = self.cloud._object_store_client.get_endpoint()
+        other_image = self.getUniqueString('no-delete')
+
+        self.register_uris([
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     service_type='object-store',
+                     resource=self.container_name,
+                     qs_elements=['format=json']),
+                 json=[{
+                     'content_type': 'application/octet-stream',
+                     'bytes': 1437258240,
+                     'hash': '249219347276c331b87bf1ac2152d9af',
+                     'last_modified': '2015-02-16T17:50:05.289600',
+                     'name': other_image,
+                 }, {
+                     'content_type': 'application/octet-stream',
+                     'bytes': 1290170880,
+                     'hash': fakes.NO_MD5,
+                     'last_modified': '2015-04-14T18:29:00.502530',
+                     'name': self.image_name,
+                 }]),
+            dict(method='HEAD',
+                 uri=self.get_mock_url(
+                     service_type='object-store',
+                     resource=self.container_name,
+                     append=[other_image]),
+                 headers={
+                     'X-Timestamp': '1429036140.50253',
+                     'X-Trans-Id': 'txbbb825960a3243b49a36f-005a0dadaedfw1',
+                     'Content-Length': '1290170880',
+                     'Last-Modified': 'Tue, 14 Apr 2015 18:29:01 GMT',
+                     'X-Object-Meta-X-Shade-Sha256': 'does not matter',
+                     'X-Object-Meta-X-Shade-Md5': 'does not matter',
+                     'Date': 'Thu, 16 Nov 2017 15:24:30 GMT',
+                     'Accept-Ranges': 'bytes',
+                     'Content-Type': 'application/octet-stream',
+                     'Etag': '249219347276c331b87bf1ac2152d9af',
+                 }),
+            dict(method='HEAD',
+                 uri=self.get_mock_url(
+                     service_type='object-store',
+                     resource=self.container_name,
+                     append=[self.image_name]),
+                 headers={
+                     'X-Timestamp': '1429036140.50253',
+                     'X-Trans-Id': 'txbbb825960a3243b49a36f-005a0dadaedfw1',
+                     'Content-Length': '1290170880',
+                     'Last-Modified': 'Tue, 14 Apr 2015 18:29:01 GMT',
+                     'X-Object-Meta-X-Shade-Sha256': fakes.NO_SHA256,
+                     'X-Object-Meta-X-Shade-Md5': fakes.NO_MD5,
+                     'Date': 'Thu, 16 Nov 2017 15:24:30 GMT',
+                     'Accept-Ranges': 'bytes',
+                     'Content-Type': 'application/octet-stream',
+                     shade.openstackcloud.OBJECT_AUTOCREATE_KEY: 'true',
+                     'Etag': fakes.NO_MD5}),
+            dict(method='DELETE',
+                 uri='{endpoint}/{container}/{object}'.format(
+                     endpoint=endpoint, container=self.container_name,
+                     object=self.image_name)),
+        ])
+
+        deleted = self.cloud.delete_autocreated_image_objects(
+            container=self.container_name)
+        self.assertTrue(deleted)
 
         self.assert_calls()
 
@@ -330,10 +484,12 @@ class TestImage(BaseTestImage):
             name, imagefile.name, wait=True, timeout=1,
             is_public=False, **kwargs)
 
-    @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
+    @mock.patch.object(shade.OpenStackCloud, '_is_client_version')
     @mock.patch.object(shade.OpenStackCloud, '_image_client')
-    def test_create_image_put_v1(self, mock_image_client, mock_api_version):
-        mock_api_version.return_value = '1'
+    def test_create_image_put_v1(
+            self, mock_image_client, mock_is_client_version):
+        # TODO(mordred) Fix this to use requests_mock
+        mock_is_client_version.return_value = False
         mock_image_client.get.return_value = []
         self.assertEqual([], self.cloud.list_images())
 
@@ -362,15 +518,15 @@ class TestImage(BaseTestImage):
                 'x-image-meta-checksum': mock.ANY,
                 'x-glance-registry-purge-props': 'false'
             })
-        mock_image_client.get.assert_called_with('/images/detail')
+        mock_image_client.get.assert_called_with('/images/detail', params={})
         self.assertEqual(
             self._munch_images(ret), self.cloud.list_images())
 
-    @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
+    @mock.patch.object(shade.OpenStackCloud, '_is_client_version')
     @mock.patch.object(shade.OpenStackCloud, '_image_client')
     def test_create_image_put_v1_bad_delete(
-            self, mock_image_client, mock_api_version):
-        mock_api_version.return_value = '1'
+            self, mock_image_client, mock_is_client_version):
+        mock_is_client_version.return_value = False
         mock_image_client.get.return_value = []
         self.assertEqual([], self.cloud.list_images())
 
@@ -404,10 +560,11 @@ class TestImage(BaseTestImage):
             })
         mock_image_client.delete.assert_called_with('/images/42')
 
-    @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
+    @mock.patch.object(shade.OpenStackCloud, '_is_client_version')
     @mock.patch.object(shade.OpenStackCloud, '_image_client')
-    def test_update_image_no_patch(self, mock_image_client, mock_api_version):
-        mock_api_version.return_value = '2'
+    def test_update_image_no_patch(
+            self, mock_image_client, mock_is_client_version):
+        mock_is_client_version.return_value = True
         self.cloud.image_api_use_tasks = False
 
         mock_image_client.get.return_value = []
@@ -431,14 +588,14 @@ class TestImage(BaseTestImage):
         self.cloud.update_image_properties(
             image=self._image_dict(ret),
             **{'owner_specified.shade.object': 'images/42 name'})
-        mock_image_client.get.assert_called_with('/images')
+        mock_image_client.get.assert_called_with('/images', params={})
         mock_image_client.patch.assert_not_called()
 
-    @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
+    @mock.patch.object(shade.OpenStackCloud, '_is_client_version')
     @mock.patch.object(shade.OpenStackCloud, '_image_client')
     def test_create_image_put_v2_bad_delete(
-            self, mock_image_client, mock_api_version):
-        mock_api_version.return_value = '2'
+            self, mock_image_client, mock_is_client_version):
+        mock_is_client_version.return_value = True
         self.cloud.image_api_use_tasks = False
 
         mock_image_client.get.return_value = []
@@ -473,11 +630,11 @@ class TestImage(BaseTestImage):
             data=mock.ANY)
         mock_image_client.delete.assert_called_with('/images/42')
 
-    @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
+    @mock.patch.object(shade.OpenStackCloud, '_is_client_version')
     @mock.patch.object(shade.OpenStackCloud, '_image_client')
     def test_create_image_put_bad_int(
-            self, mock_image_client, mock_api_version):
-        mock_api_version.return_value = '2'
+            self, mock_image_client, mock_is_client_version):
+        mock_is_client_version.return_value = True
         self.cloud.image_api_use_tasks = False
 
         self.assertRaises(
@@ -485,11 +642,11 @@ class TestImage(BaseTestImage):
             self._call_create_image, '42 name', min_disk='fish', min_ram=0)
         mock_image_client.post.assert_not_called()
 
-    @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
+    @mock.patch.object(shade.OpenStackCloud, '_is_client_version')
     @mock.patch.object(shade.OpenStackCloud, '_image_client')
     def test_create_image_put_user_int(
-            self, mock_image_client, mock_api_version):
-        mock_api_version.return_value = '2'
+            self, mock_image_client, mock_is_client_version):
+        mock_is_client_version.return_value = True
         self.cloud.image_api_use_tasks = False
 
         args = {'name': '42 name',
@@ -516,15 +673,15 @@ class TestImage(BaseTestImage):
             '/images/42/file',
             headers={'Content-Type': 'application/octet-stream'},
             data=mock.ANY)
-        mock_image_client.get.assert_called_with('/images')
+        mock_image_client.get.assert_called_with('/images', params={})
         self.assertEqual(
             self._munch_images(ret), self.cloud.list_images())
 
-    @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
+    @mock.patch.object(shade.OpenStackCloud, '_is_client_version')
     @mock.patch.object(shade.OpenStackCloud, '_image_client')
     def test_create_image_put_meta_int(
-            self, mock_image_client, mock_api_version):
-        mock_api_version.return_value = '2'
+            self, mock_image_client, mock_is_client_version):
+        mock_is_client_version.return_value = True
         self.cloud.image_api_use_tasks = False
 
         mock_image_client.get.return_value = []
@@ -545,15 +702,15 @@ class TestImage(BaseTestImage):
         ret['status'] = 'success'
         mock_image_client.get.return_value = [ret]
         mock_image_client.post.return_value = ret
-        mock_image_client.get.assert_called_with('/images')
+        mock_image_client.get.assert_called_with('/images', params={})
         self.assertEqual(
             self._munch_images(ret), self.cloud.list_images())
 
-    @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
+    @mock.patch.object(shade.OpenStackCloud, '_is_client_version')
     @mock.patch.object(shade.OpenStackCloud, '_image_client')
     def test_create_image_put_protected(
-            self, mock_image_client, mock_api_version):
-        mock_api_version.return_value = '2'
+            self, mock_image_client, mock_is_client_version):
+        mock_is_client_version.return_value = True
         self.cloud.image_api_use_tasks = False
 
         mock_image_client.get.return_value = []
@@ -587,11 +744,11 @@ class TestImage(BaseTestImage):
             headers={'Content-Type': 'application/octet-stream'})
         self.assertEqual(self._munch_images(ret), self.cloud.list_images())
 
-    @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
+    @mock.patch.object(shade.OpenStackCloud, '_is_client_version')
     @mock.patch.object(shade.OpenStackCloud, '_image_client')
     def test_create_image_put_user_prop(
-            self, mock_image_client, mock_api_version):
-        mock_api_version.return_value = '2'
+            self, mock_image_client, mock_is_client_version):
+        mock_is_client_version.return_value = True
         self.cloud.image_api_use_tasks = False
 
         mock_image_client.get.return_value = []
@@ -613,10 +770,60 @@ class TestImage(BaseTestImage):
         mock_image_client.post.return_value = ret
         self._call_create_image(
             '42 name', min_disk='0', min_ram=0, properties={'int_v': 12345})
-        mock_image_client.get.assert_called_with('/images')
+        mock_image_client.get.assert_called_with('/images', params={})
         self.assertEqual(
             self._munch_images(ret), self.cloud.list_images())
 
+    def test_get_image_by_id(self):
+        self.register_uris([
+            dict(method='GET',
+                 uri='https://image.example.com/v2/images/{id}'.format(
+                     id=self.image_id),
+                 json=self.fake_image_dict)
+        ])
+        self.assertEqual(
+            self.cloud._normalize_image(self.fake_image_dict),
+            self.cloud.get_image_by_id(self.image_id))
+        self.assert_calls()
+
+
+class TestImageSuburl(BaseTestImage):
+
+    def setUp(self):
+        super(TestImageSuburl, self).setUp()
+        self.use_keystone_v3(catalog='catalog-v3-suburl.json')
+        self.use_glance(
+            image_version_json='image-version-suburl.json',
+            image_discovery_url='https://example.com/image')
+
+    def test_list_images(self):
+        self.register_uris([
+            dict(method='GET', uri='https://example.com/image/v2/images',
+                 json=self.fake_search_return)
+        ])
+        self.assertEqual(
+            self.cloud._normalize_images([self.fake_image_dict]),
+            self.cloud.list_images())
+        self.assert_calls()
+
+    def test_list_images_paginated(self):
+        marker = str(uuid.uuid4())
+        self.register_uris([
+            dict(method='GET', uri='https://example.com/image/v2/images',
+                 json={'images': [self.fake_image_dict],
+                       'next': '/v2/images?marker={marker}'.format(
+                           marker=marker)}),
+            dict(method='GET',
+                 uri=('https://example.com/image/v2/images?'
+                      'marker={marker}'.format(marker=marker)),
+                 json=self.fake_search_return)
+        ])
+        self.assertEqual(
+            self.cloud._normalize_images([
+                self.fake_image_dict, self.fake_image_dict]),
+            self.cloud.list_images())
+        self.assert_calls()
+
 
 class TestImageV1Only(base.RequestsMockTestCase):
 
@@ -632,8 +839,7 @@ class TestImageV1Only(base.RequestsMockTestCase):
         self.assertEqual(
             'https://image.example.com/v1/',
             self.cloud._image_client.get_endpoint())
-        self.assertEqual(
-            '1', self.cloud_config.get_api_version('image'))
+        self.assertTrue(self.cloud._is_client_version('image', 1))
 
     def test_config_v2(self):
         self.cloud.cloud_config.config['image_api_version'] = '2'
@@ -642,8 +848,7 @@ class TestImageV1Only(base.RequestsMockTestCase):
         self.assertEqual(
             'https://image.example.com/v1/',
             self.cloud._image_client.get_endpoint())
-        self.assertEqual(
-            '1', self.cloud_config.get_api_version('image'))
+        self.assertFalse(self.cloud._is_client_version('image', 2))
 
 
 class TestImageV2Only(base.RequestsMockTestCase):
@@ -659,8 +864,7 @@ class TestImageV2Only(base.RequestsMockTestCase):
         self.assertEqual(
             'https://image.example.com/v2/',
             self.cloud._image_client.get_endpoint())
-        self.assertEqual(
-            '2', self.cloud_config.get_api_version('image'))
+        self.assertTrue(self.cloud._is_client_version('image', 2))
 
     def test_config_v2(self):
         self.cloud.cloud_config.config['image_api_version'] = '2'
@@ -669,26 +873,7 @@ class TestImageV2Only(base.RequestsMockTestCase):
         self.assertEqual(
             'https://image.example.com/v2/',
             self.cloud._image_client.get_endpoint())
-        self.assertEqual(
-            '2', self.cloud_config.get_api_version('image'))
-
-
-class TestImageVersionDiscovery(BaseTestImage):
-
-    def test_version_discovery_skip(self):
-        self.cloud.cloud_config.config['image_endpoint_override'] = \
-            'https://image.example.com/v2/override'
-
-        self.register_uris([
-            dict(method='GET',
-                 uri='https://image.example.com/v2/override/images',
-                 json={'images': []})
-        ])
-        self.assertEqual([], self.cloud.list_images())
-        self.assertEqual(
-            self.cloud._image_client.endpoint_override,
-            'https://image.example.com/v2/override')
-        self.assert_calls()
+        self.assertTrue(self.cloud._is_client_version('image', 2))
 
 
 class TestImageVolume(BaseTestImage):
@@ -774,24 +959,6 @@ class TestImageBrokenDiscovery(base.RequestsMockTestCase):
         ])
         self.assertEqual([], self.cloud.list_images())
         self.assertEqual(
-            self.cloud._image_client.endpoint_override,
+            self.cloud._image_client.get_endpoint(),
             'https://image.example.com/v2/')
         self.assert_calls()
-
-
-class TestImageDiscoveryOptimization(base.RequestsMockTestCase):
-
-    def setUp(self):
-        super(TestImageDiscoveryOptimization, self).setUp()
-        self.use_keystone_v3(catalog='catalog-versioned-image.json')
-
-    def test_version_discovery_skip(self):
-        self.cloud.cloud_config.config['image_api_version'] = '2'
-
-        self.register_uris([
-            dict(method='GET',
-                 uri='https://image.example.com/v2/images',
-                 json={'images': []})
-        ])
-        self.assertEqual([], self.cloud.list_images())
-        self.assert_calls()
